#!/usr/bin/perl -w

use strict;
use File::Basename;

my %Conf = (
    software_name => "iSPY 0.1",
    direct_sending_download => 1, #directly send file after filling out the form? 1: yes, 0: no, wait for admin to respond
    file_location => "_iSPY_0.1.tar.gz", #this parameter is useless if !$direct_sending_download
    file_appr_name => "iSPY_0.1.tar.gz", 
    send_email => 1, #send email to admin after the form has been filled out? 1: yes, 0: no
    my_email => 'zhang97@purdue.edu', #admin's email, it's useless if !send_email
    );

my %Para = ( 
    name => { check => sub { $_[0] eq '' ? "This field is required!" : '' } },
    email => { check => sub { $_[0] eq '' ? "This field is required!" : '' } },
    company => { check => sub { $_[0] eq '' ? "This field is required!" : '' } },
    purpose => { check => sub { $_[0] eq '' ? "This field is required!" : '' } },
    captcha => { check => sub { $_[0] ne "GH15LK" ? "Character mismatch!" : '' } },
    );

sub GenPage ($$) {
    my ($software_name, $Para) = @_;
    
    my $html = <<END;
<HTML>
<HEAD>
  <TITLE>Downloading $software_name</TITLE>
  <meta name="author" content="Zheng Zhang">
  <link rel="stylesheet" href="../style.css" type="text/css">
</HEAD>
<BODY>
<H2>Downloading $software_name</H2>

Please kindly provide the following information which are used by us to keep track of the
download and intended usage of our software.<P>

<FORM method=POST>
Name:<BR>
<INPUT NAME=name VALUE="$Para->{name}{value}" SIZE=80> $Para->{name}{error}<BR>
Email: (please use your professional email, not your offshore account such as yahoo):<br>
<INPUT NAME=email VALUE="$Para->{email}{value}" SIZE=80> $Para->{email}{error}<BR>
Affiliation: <BR>
<INPUT NAME=company VALUE="$Para->{company}{value}" SIZE=80> $Para->{company}{error}<BR>
A short description of what you will use $software_name for: <BR>
<TEXTAREA NAME=purpose COLS=60 ROWS=10 WRAP=auto>$Para->{purpose}{value}</TEXTAREA> $Para->{purpose}{error}<BR>
<P>
Type the characters shown in the picture below:<BR>
<INPUT NAME=captcha VALUE="$Para->{captcha}{value}" SIZE=10> $Para->{captcha}{error}<BR>
<IMG SRC="CAPTCHA_image.gif"><BR>
<P>
<INPUT TYPE=SUBMIT NAME=Download VALUE="Download">
</FORM>
</BODY>
</HTML>
END
    return $html;
}

sub send_email($$$$) {
    my $sendmail = "/usr/sbin/sendmail -t";
    my $from = "From: $_[0]\n";
    my $reply_to = "Reply-to: $_[0]\n";
    my $send_to  = "To: $_[1]\n";
    my $subject  = "Subject: $_[2]\n";
    my $content  = $_[3];
    
    open(SENDMAIL, "|$sendmail") or die "Cannot open $sendmail: $!";
#    open(SENDMAIL, ">>dl.log") or die "Cannot open $sendmail: $!";
    print SENDMAIL $from;
    print SENDMAIL $reply_to;
    print SENDMAIL $subject;
    print SENDMAIL $send_to;
    print SENDMAIL "Content-type: text/plain\n\n";
    print SENDMAIL $content;
#    print SENDMAIL "\nGenerated by ", $ENV{'REQUEST_URI'}, " , ", `date`;
    close(SENDMAIL);
}

sub GetParas ($) {
    my ($Para) = @_;

    my $line;
    read(STDIN, $line, $ENV{'CONTENT_LENGTH'});
    my @values = split(/&/, $line);
    
    foreach my $i (@values) {
        my ($varname, $data) = split(/=/, $i);
        $data =~ s/\@/\\\@/g;
        $data =~ tr/+/ /; 
        $data =~ s/%(..)/pack("C", hex($1))/eg;
        $Para->{$varname}{value} = $data;
    }
}

GetParas(\%Para);

#print "Content-Type: text/html; charset=ISO-8859-1\n\n";
#foreach my $varname (keys %Para) { print "$varname\n"; }

#Sanity check
my $sanity_pass = 1;

if ($ENV{'REQUEST_METHOD'} eq "POST") {
    foreach my $varname (keys %Para) {
        if (exists $Para{$varname}{check}) {
            my $ret = &{$Para{$varname}{check}}( $Para{$varname}{value} );
            if ($ret ne '') {
                $Para{$varname}{error} = '<FONT color=red>'.$ret.'</FONT>';
                $sanity_pass = 0;
            }
        }
    }
}

if ($ENV{'REQUEST_METHOD'} ne "POST" || !$sanity_pass) {
#print "my $f_name_error; my $f_email_error; my $f_company_error; my $f_purpose_error;";
    print "Content-Type: text/html; charset=ISO-8859-1\n\n";
    print GenPage($Conf{software_name}, \%Para);
}
else {#Otherwise send the file for download or display thank-you message
    my $msg = join ("\n", map { $_ . ": " . $Para{$_}{value} } (keys %Para));
    $msg = `date` . $msg . "\nRequest from host: $ENV{REMOTE_HOST}\n\n";
    open (LOG, ">>dl.log") || Error('open', 'file');
    print LOG $msg;
    close (LOG);
    
    if ($Conf{send_email}) {
        send_email ($Conf{my_email}, $Conf{my_email}, 
                    "Download request of $Conf{software_name}",
                    $msg);
    }
    
    if ($Conf{direct_sending_download}) {
        open (my $DLFILE, "<$Conf{file_location}") || Error('open', 'file');  
        my @fileholder = <$DLFILE>;  
        close ($DLFILE) || Error ('close', 'file');
        print "Content-Type: application/x-download\n";
        print "Content-Disposition: attachment; filename=$Conf{file_appr_name}\n\n";
        print @fileholder ;
    }
    else { #Print thank-you message
        print "Content-Type: text/html; charset=ISO-8859-1\n\n";
        my $message = <<END;
<HTML>
<TITLE>Downloading $Conf{software_name}</TITLE>
<BODY>
We have received your download request and will respond to you via email shortly. Thank you for your interest in $Conf{software_name}.
</BODY>
</HTML>
END
        print $message;
    }
}
